import openai
import requests
from typing import List, Dict
from retry.api import retry_call
from openai.error import RateLimitError
from openai.error import APIError
from openai.error import TryAgain
from openai.error import Timeout
from openai.error import APIConnectionError
from openai.error import ServiceUnavailableError
from openai.error import InvalidRequestError
from key import API_KEY



DEFAULT_API_MAX_RETRIES = 3
DEFAULT_API_DELAY_TIME = 5
API_RETRY_EXCEPTIONS = (
    requests.HTTPError,
    RateLimitError,
    APIError,
    TryAgain,
    Timeout,
    APIConnectionError,
    ServiceUnavailableError)
API_CATCH_EXCEPTIONS = (
    requests.HTTPError,
    RateLimitError,
    APIError,
    TryAgain,
    Timeout,
    APIConnectionError,
    ServiceUnavailableError,
    InvalidRequestError)


def send_prompt_with_context(model: str,
                             messages: List[Dict],
                             temperature: int = 0.1,
                             max_tokens: int = 500) -> Dict[str, str]:
    openai.api_key = API_KEY
    resp = openai.ChatCompletion.create(
        messages=messages,
        model=model,
        temperature=temperature,
        max_tokens=max_tokens
    )
    print(resp)
    return resp.choices[0].message["content"]


def predict_entity(model: str, input_text: str):
    def predict_entity_values() -> Dict[str, str]:

        system_msg = """
        Definition: Capture entity values from last utterance of the conversation according to examples.
        Capture pair "entity:value" seperated by colon and no spaces in between. Seperate entity:value pairs by hyphens.
        If not specified, leave the value empty. Values that should be captures are:
        - "pricerange": the price of the hotel
        - "area" that specifies the area where the hotel is located (north/east/south/west,centre)
        - "internet" that specifies if the hotel has internet (yes/no)
        - "parking" that specifies if the hotel has parking (yes/no)
        - "stars" that specifies the number of stars of the hotel (1/2/3/4/5)
        - "type" that specifies the type of the hotel (hotel/bed and breakfast/guest house)
        """
        # Extract last utterance
        utterance = input_text.strip().split('\n')[-1]

        messages = [
            {"role": "system", "content": system_msg},
            {"role": "user", "content": f"Customer: {utterance}"}
        ]

        answer = send_prompt_with_context(model, messages)
        print(answer)
        return answer
    return retry_call(predict_entity_values,
                      exceptions=API_RETRY_EXCEPTIONS,
                      delay=DEFAULT_API_DELAY_TIME,
                      tries=DEFAULT_API_MAX_RETRIES)
if __name__ == "__main__":
    model = "gpt-3.5-turbo"
    input_text = "I am looking for a hotel in the north area of town."
    print(predict_entity(model, input_text))